@@ -3,8 +3,11 @@
 //! This module provides functionality for searching files in Debian
 //! packages, including using apt-file and other package contents databases.
 
-use crate::debian::sources_list::{SourcesEntry, SourcesList};
 use crate::session::{Error as SessionError, Session};
+use apt_sources::{
+    error::{LoadError, RepositoryError},
+    Repository, RepositoryType,
+};
 use debian_control::apt::Release;
 use flate2::read::GzDecoder;
 use lzma_rs::lzma_decompress;
@@ -33,6 +36,18 @@ impl From<std::io::Error> for Error {
     }
 }
 
+impl From<RepositoryError> for Error {
+    fn from(e: RepositoryError) -> Error {
+        Error::AptFileAccessError(format!("Repository error: {}", e))
+    }
+}
+
+impl From<LoadError> for Error {
+    fn from(e: LoadError) -> Error {
+        Error::AptFileAccessError(format!("Load error: {}", e))
+    }
+}
+
 impl std::fmt::Display for Error {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         match self {
@@ -97,23 +112,36 @@ pub fn read_contents_file<R: Read>(f: R) -> impl Iterator<Item = (String, String
     })
 }
 
-/// Get URLs for contents files from a sources entry.
+/// Get URLs for contents files from a repository entry.
 ///
 /// # Arguments
-/// * `entry` - Sources entry to get contents URLs from
+/// * `repo` - Repository to get contents URLs from
 /// * `arches` - List of architectures to include
 /// * `load_url` - Function to load a URL and get a reader
 ///
 /// # Returns
 /// Iterator of URLs for contents files
-pub fn contents_urls_from_sources_entry<'a>(
-    entry: &'a SourcesEntry,
+pub fn contents_urls_from_repository<'a>(
+    repo: &'a Repository,
     arches: Vec<&'a str>,
     load_url: impl Fn(&url::Url) -> Result<Box<dyn Read>, Error>,
 ) -> Box<dyn Iterator<Item = url::Url> + 'a> {
-    match entry {
-        SourcesEntry::Deb { uri, dist, comps } => {
-            let base_url = uri.trim_end_matches('/');
+    // Only process binary repositories (deb), not source repositories (deb-src)
+    if !repo.types.contains(&RepositoryType::Binary) {
+        return Box::new(vec![].into_iter());
+    }
+
+    // Process all URIs and suites combinations
+    let mut all_urls = Vec::new();
+
+    for uri in &repo.uris {
+        for dist in &repo.suites {
+            let comps = repo
+                .components
+                .as_ref()
+                .map(|c| c.as_slice())
+                .unwrap_or(&[]);
+            let base_url = uri.as_str().trim_end_matches('/');
             let name = dist.trim_end_matches('/');
             let dists_url: url::Url = if comps.is_empty() {
                 base_url.to_string()
@@ -163,7 +191,7 @@ pub fn contents_urls_from_sources_entry<'a>(
             }
             let mut contents_files = HashSet::new();
             if comps.is_empty() {
-                for arch in arches {
+                for arch in &arches {
                     contents_files.insert(format!("Contents-{}", arch));
                 }
             } else {
@@ -173,37 +201,43 @@ pub fn contents_urls_from_sources_entry<'a>(
                     }
                 }
             }
-            return Box::new(contents_files.into_iter().filter_map(move |f| {
-                if let Some(name) =
-                    existing_names.get(&std::path::Path::new(&f).file_stem().unwrap().to_owned())
-                {
-                    return Some(dists_url.join(name).unwrap().join(&f).unwrap());
-                }
-                None
-            }));
+            let urls: Vec<_> = contents_files
+                .into_iter()
+                .filter_map(move |f| {
+                    if let Some(name) = existing_names
+                        .get(&std::path::Path::new(&f).file_stem().unwrap().to_owned())
+                    {
+                        return Some(dists_url.join(name).unwrap().join(&f).unwrap());
+                    }
+                    None
+                })
+                .collect();
+            all_urls.extend(urls);
         }
-        SourcesEntry::DebSrc { .. } => Box::new(vec![].into_iter()),
     }
+
+    Box::new(all_urls.into_iter())
 }
 
-/// Get URLs for contents files from a sources.list file.
+/// Get URLs for contents files from APT sources.
 ///
 /// # Arguments
-/// * `sl` - Sources list to get contents URLs from
+/// * `repositories` - Repositories to get contents URLs from
 /// * `arch` - Architecture to include
 /// * `load_url` - Function to load a URL and get a reader
 ///
 /// # Returns
 /// Iterator of URLs for contents files
-pub fn contents_urls_from_sourceslist<'a>(
-    sl: &'a SourcesList,
+pub fn contents_urls_from_sources<'a>(
+    repositories: &'a apt_sources::Repositories,
     arch: &'a str,
     load_url: impl Fn(&'_ url::Url) -> Result<Box<dyn Read>, Error> + 'a + Copy,
 ) -> impl Iterator<Item = url::Url> + 'a {
     // TODO(jelmer): Verify signatures, etc.
     let arches = vec![arch, "all"];
-    sl.iter()
-        .flat_map(move |source| contents_urls_from_sources_entry(source, arches.clone(), load_url))
+    repositories
+        .iter()
+        .flat_map(move |repo| contents_urls_from_repository(repo, arches.clone(), load_url))
 }
 
 /// Unwrap a compressed file based on its extension.
@@ -220,8 +254,9 @@ pub fn unwrap<'a, R: Read + 'a>(f: R, ext: &str) -> Result<Box<dyn Read + 'a>, E
         ".xz" => {
             let mut compressed_reader = BufReader::new(f);
             let mut decompressed_data = Vec::new();
-            lzma_decompress(&mut compressed_reader, &mut decompressed_data)
-                .map_err(|e| Error::DecompressionError(format!("LZMA decompression failed: {}", e)))?;
+            lzma_decompress(&mut compressed_reader, &mut decompressed_data).map_err(|e| {
+                Error::DecompressionError(format!("LZMA decompression failed: {}", e))
+            })?;
             Ok(Box::new(std::io::Cursor::new(decompressed_data)))
         }
         ".lz4" => Ok(Box::new(lz4_flex::frame::FrameDecoder::new(f))),
@@ -237,13 +272,21 @@ pub fn unwrap<'a, R: Read + 'a>(f: R, ext: &str) -> Result<Box<dyn Read + 'a>, E
 /// # Returns
 /// Reader for the URL contents
 pub fn load_direct_url(url: &url::Url) -> Result<Box<dyn Read>, Error> {
+    // Create a client with reasonable timeouts for downloading large APT Contents files
+    let client = reqwest::blocking::Client::builder()
+        .timeout(std::time::Duration::from_secs(300)) // 5 minutes for large files
+        .connect_timeout(std::time::Duration::from_secs(30)) // 30 seconds to establish connection
+        .build()
+        .map_err(|e| Error::AptFileAccessError(format!("Failed to create HTTP client: {}", e)))?;
+
     for ext in [".xz", ".gz", ""] {
-        let response = match reqwest::blocking::get(url.to_string() + ext) {
+        let response = match client.get(url.to_string() + ext).send() {
             Ok(response) => response,
             Err(e) => {
                 if e.status() == Some(reqwest::StatusCode::NOT_FOUND) {
                     continue;
                 }
+                log::debug!("Failed to fetch APT contents from {}{}: {}", url, ext, e);
                 return Err(Error::AptFileAccessError(format!(
                     "Unable to access apt URL {}{}: {}",
                     url, ext, e
@@ -255,6 +298,11 @@ pub fn load_direct_url(url: &url::Url) -> Result<Box<dyn Read>, Error> {
     Err(Error::FileNotFoundError(format!("{} not found", url)))
 }
 
+/// Get the user cache directory for ognibuild APT Contents files.
+fn get_user_cache_dir() -> Option<PathBuf> {
+    dirs::cache_dir().map(|d| d.join("ognibuild").join("apt-contents"))
+}
+
 /// Load a URL with caching in the specified directories.
 ///
 /// # Arguments
@@ -264,6 +312,7 @@ pub fn load_direct_url(url: &url::Url) -> Result<Box<dyn Read>, Error> {
 /// # Returns
 /// Reader for the URL contents
 pub fn load_url_with_cache(url: &url::Url, cache_dirs: &[&Path]) -> Result<Box<dyn Read>, Error> {
+    // First check system cache directories
     for cache_dir in cache_dirs {
         match load_apt_cache_file(url, cache_dir) {
             Ok(f) => return Ok(Box::new(f)),
@@ -274,7 +323,74 @@ pub fn load_url_with_cache(url: &url::Url, cache_dirs: &[&Path]) -> Result<Box<d
             }
         }
     }
-    load_direct_url(url)
+
+    // Then check user cache directory
+    if let Some(user_cache_dir) = get_user_cache_dir() {
+        match load_apt_cache_file(url, &user_cache_dir) {
+            Ok(f) => {
+                log::debug!(
+                    "Found cached APT contents in user cache: {}",
+                    user_cache_dir.display()
+                );
+                return Ok(Box::new(f));
+            }
+            Err(e) => {
+                if e.kind() != std::io::ErrorKind::NotFound {
+                    log::debug!("Error reading from user cache: {}", e);
+                }
+            }
+        }
+    }
+
+    // If not found in any cache, download and cache it
+    download_and_cache_url(url)
+}
+
+/// Download a URL and cache it in the user cache directory.
+fn download_and_cache_url(url: &url::Url) -> Result<Box<dyn Read>, Error> {
+    // Download the file
+    let content = load_direct_url(url)?;
+
+    // Try to cache it in user directory
+    if let Some(user_cache_dir) = get_user_cache_dir() {
+        // Ensure cache directory exists
+        if let Err(e) = std::fs::create_dir_all(&user_cache_dir) {
+            log::debug!(
+                "Failed to create cache directory {}: {}",
+                user_cache_dir.display(),
+                e
+            );
+        } else {
+            // Read the content into memory so we can both cache and return it
+            let mut buffer = Vec::new();
+            let mut reader = content;
+            if let Err(e) = std::io::Read::read_to_end(&mut reader, &mut buffer) {
+                log::debug!("Failed to read content for caching: {}", e);
+                return Ok(reader); // Return the original reader if we can't cache
+            }
+
+            // Write to cache file
+            let cache_file_path = user_cache_dir.join(uri_to_filename(url));
+            match std::fs::write(&cache_file_path, &buffer) {
+                Ok(_) => {
+                    log::info!("Cached APT contents to: {}", cache_file_path.display());
+                }
+                Err(e) => {
+                    log::debug!(
+                        "Failed to write cache file {}: {}",
+                        cache_file_path.display(),
+                        e
+                    );
+                }
+            }
+
+            // Return the buffer as a reader
+            return Ok(Box::new(std::io::Cursor::new(buffer)));
+        }
+    }
+
+    // If we can't cache, just return the downloaded content
+    Ok(content)
 }
 
 /// Convert a URI into a safe filename. It quotes all unsafe characters and converts / to _ and removes the scheme identifier.
@@ -344,7 +460,9 @@ pub fn load_apt_cache_file(
         let f = File::open(p)?;
         return unwrap(f, ext).map_err(|e| match e {
             Error::IoError(io_err) => io_err,
-            Error::DecompressionError(msg) => std::io::Error::new(std::io::ErrorKind::InvalidData, msg),
+            Error::DecompressionError(msg) => {
+                std::io::Error::new(std::io::ErrorKind::InvalidData, msg)
+            }
             other => std::io::Error::new(std::io::ErrorKind::Other, format!("{:?}", other)),
         });
     }
@@ -402,7 +520,7 @@ impl<'a> AptFileFileSearcher<'a> {
     ///
     /// # Returns
     /// A new AptFileFileSearcher instance
-    pub fn from_session(session: &dyn Session) -> AptFileFileSearcher {
+    pub fn from_session(session: &dyn Session) -> AptFileFileSearcher<'_> {
         log::debug!("Using apt-file to search apt contents");
         if !session.exists(&CACHE_IS_EMPTY_PATH) {
             crate::debian::apt::AptManager::from_session(session)
@@ -541,6 +659,44 @@ impl<'b> FileSearcher<'b> for AptFileFileSearcher<'b> {
     }
 }
 
+/// Set up apt-file in a session.
+///
+/// This function installs apt-file if needed and updates the apt-file cache.
+///
+/// # Arguments
+/// * `session` - Session to set up
+///
+/// # Returns
+/// Ok(()) if setup was successful, Error otherwise
+pub fn setup_apt_file(session: &dyn Session) -> Result<(), Error> {
+    // Update APT package lists first
+    log::info!("Updating APT package lists...");
+    session
+        .command(vec!["apt-get", "update"])
+        .user("root")
+        .check_call()
+        .map_err(|e| Error::AptFileAccessError(format!("Failed to run apt-get update: {}", e)))?;
+
+    // Install apt-file if not already installed
+    log::info!("Installing apt-file...");
+    session
+        .command(vec!["apt-get", "install", "-y", "apt-file"])
+        .user("root")
+        .check_call()
+        .map_err(|e| Error::AptFileAccessError(format!("Failed to install apt-file: {}", e)))?;
+
+    // Update apt-file cache
+    log::info!("Updating apt-file cache...");
+    session
+        .command(vec!["apt-file", "update"])
+        .user("root")
+        .check_call()
+        .map_err(|e| Error::AptFileAccessError(format!("Failed to run apt-file update: {}", e)))?;
+
+    log::info!("apt-file setup complete");
+    Ok(())
+}
+
 /// Get a file searcher that uses apt-file or remote contents.
 ///
 /// This function returns the appropriate file searcher based on whether
@@ -558,8 +714,9 @@ pub fn get_apt_contents_file_searcher<'a>(
     if AptFileFileSearcher::has_cache(session).unwrap() {
         Ok(Box::new(AptFileFileSearcher::from_session(session)) as Box<dyn FileSearcher<'a>>)
     } else {
-        Ok(Box::new(RemoteContentsFileSearcher::from_session(session)?)
-            as Box<dyn FileSearcher<'a>>)
+        // Try to load remote contents, but with timeouts to prevent hanging
+        RemoteContentsFileSearcher::from_session(session)
+            .map(|searcher| Box::new(searcher) as Box<dyn FileSearcher<'a>>)
     }
 }
 
@@ -595,12 +752,12 @@ impl RemoteContentsFileSearcher {
     /// # Returns
     /// Ok(()) if successful, Error otherwise
     pub fn load_local(&mut self) -> Result<(), Error> {
-        let sl = SourcesList::default();
+        let repositories = apt_sources::Repositories::default();
         let arch = crate::debian::build::get_build_architecture();
         let cache_dirs = vec![Path::new("/var/lib/apt/lists")];
         let load_url = |url: &url::Url| load_url_with_cache(url, cache_dirs.as_slice());
-        let urls = contents_urls_from_sourceslist(&sl, &arch, load_url);
-        self.load_urls(urls, load_url)
+        let urls = contents_urls_from_sources(&repositories, &arch, load_url);
+        self.load_urls(urls, load_url, false)
     }
 
     /// Load contents information from APT sources configured in a session.
@@ -611,8 +768,9 @@ impl RemoteContentsFileSearcher {
     /// # Returns
     /// Ok(()) if successful, Error otherwise
     pub fn load_from_session(&mut self, session: &dyn Session) -> Result<(), Error> {
-        // TODO(jelmer): what about sources.list.d?
-        let sl = SourcesList::from_apt_dir(&session.external_path(Path::new("/etc/apt")));
+        let (repositories, _errors) = apt_sources::Repositories::load_from_directory(
+            &session.external_path(Path::new("/etc/apt")),
+        );
         let arch = crate::debian::build::get_build_architecture();
         let cache_dirs = [session.external_path(Path::new("/var/lib/apt/lists"))];
         let load_url = |url: &url::Url| {
@@ -625,8 +783,8 @@ impl RemoteContentsFileSearcher {
                     .as_slice(),
             )
         };
-        let urls = contents_urls_from_sourceslist(&sl, &arch, load_url);
-        self.load_urls(urls, load_url)
+        let urls = contents_urls_from_sources(&repositories, &arch, load_url);
+        self.load_urls(urls, load_url, false)
     }
 
     /// Load contents information from multiple URLs.
@@ -641,11 +799,77 @@ impl RemoteContentsFileSearcher {
         &mut self,
         urls: impl Iterator<Item = url::Url>,
         load_url: impl Fn(&url::Url) -> Result<Box<dyn Read>, Error>,
+        fail_on_error: bool,
     ) -> Result<(), Error> {
-        for url in urls {
-            let f = load_url(&url)?;
-            self.load_file(f, url);
+        let urls: Vec<url::Url> = urls.collect();
+        let num_urls = urls.len();
+
+        if num_urls == 0 {
+            return Ok(());
         }
+
+        log::info!(
+            "Loading {} APT Contents files (this may take several minutes)...",
+            num_urls
+        );
+
+        let mut success_count = 0;
+        let mut contents = Vec::new();
+
+        // Try to load each URL
+        for (idx, url) in urls.iter().enumerate() {
+            log::info!("Loading Contents file {}/{}: {}", idx + 1, num_urls, url);
+            match load_url(&url) {
+                Ok(reader) => {
+                    // Read the entire content into memory
+                    let mut content = Vec::new();
+                    let mut reader = reader;
+                    match std::io::Read::read_to_end(&mut reader, &mut content) {
+                        Ok(size) => {
+                            log::info!("Successfully loaded {} bytes from {}", size, url);
+                            contents.push((url.clone(), content));
+                            success_count += 1;
+                        }
+                        Err(e) => {
+                            if fail_on_error {
+                                return Err(Error::AptFileAccessError(format!(
+                                    "Failed to read Contents from {}: {}",
+                                    url, e
+                                )));
+                            } else {
+                                log::warn!("Failed to read Contents from {}: {}", url, e);
+                            }
+                        }
+                    }
+                }
+                Err(e) => {
+                    if fail_on_error {
+                        return Err(e);
+                    } else {
+                        log::warn!("Failed to load Contents from {}: {}", url, e);
+                    }
+                }
+            }
+        }
+
+        log::info!(
+            "Successfully loaded {}/{} Contents files",
+            success_count,
+            num_urls
+        );
+
+        if success_count == 0 {
+            return Err(Error::AptFileAccessError(
+                "Failed to download any APT Contents files".to_string(),
+            ));
+        }
+
+        // Process the successfully loaded files
+        for (url, content) in contents {
+            let reader = Box::new(std::io::Cursor::new(content));
+            self.load_file(reader, url);
+        }
+
         Ok(())
     }
 
@@ -1059,12 +1283,12 @@ mod tests {
         use flate2::write::GzEncoder;
         use flate2::Compression;
         use std::io::Write;
-        
+
         let original = b"hello world from gzip";
         let mut encoder = GzEncoder::new(Vec::new(), Compression::default());
         encoder.write_all(original).unwrap();
         let compressed = encoder.finish().unwrap();
-        
+
         let f = std::io::Cursor::new(compressed);
         let mut f = unwrap(f, ".gz").unwrap();
         let mut buf = Vec::new();
@@ -1075,11 +1299,11 @@ mod tests {
     #[test]
     fn test_unwrap_xz() {
         use lzma_rs::lzma_compress;
-        
+
         let original = b"hello world from xz";
         let mut compressed = Vec::new();
         lzma_compress(&mut original.as_ref(), &mut compressed).unwrap();
-        
+
         let f = std::io::Cursor::new(compressed);
         let mut f = unwrap(f, ".xz").unwrap();
         let mut buf = Vec::new();
@@ -1100,4 +1324,61 @@ mod tests {
             panic!("Expected DecompressionError");
         }
     }
+
+    #[test]
+    fn test_setup_apt_file() {
+        use crate::session::unshare::{create_debian_session_for_testing, UnshareSession};
+
+        fn test_session() -> Option<UnshareSession> {
+            // Don't run tests if we're in github actions (CI environment restrictions)
+            if std::env::var("GITHUB_ACTIONS").is_ok() {
+                return None;
+            }
+            create_debian_session_for_testing("sid").ok()
+        }
+
+        let session = if let Some(session) = test_session() {
+            session
+        } else {
+            return;
+        };
+
+        // Test that setup_apt_file runs without errors
+        let result = setup_apt_file(&session);
+        assert!(result.is_ok(), "setup_apt_file failed: {:?}", result);
+
+        // Verify apt-file is installed and functional
+        let output = session
+            .command(vec!["apt-file", "--help"])
+            .output()
+            .expect("Failed to run apt-file --help");
+        assert!(output.status.success(), "apt-file --help failed");
+
+        // Verify apt-file cache exists (Contents files should be downloaded)
+        let cache_check = session
+            .command(vec!["ls", "/var/cache/apt/apt-file/"])
+            .output()
+            .expect("Failed to check apt-file cache");
+        assert!(
+            cache_check.status.success(),
+            "apt-file cache directory not found"
+        );
+
+        // Test that apt-file can actually search for a file
+        let search_result = session
+            .command(vec!["apt-file", "search", "bin/ls"])
+            .output()
+            .expect("Failed to run apt-file search");
+        assert!(search_result.status.success(), "apt-file search failed");
+
+        let search_output = String::from_utf8_lossy(&search_result.stdout);
+        assert!(
+            !search_output.trim().is_empty(),
+            "apt-file search returned no results"
+        );
+        assert!(
+            search_output.contains("coreutils"),
+            "Expected coreutils package in search results"
+        );
+    }
 }
