@@ -11,6 +11,33 @@ pub mod schroot;
 #[cfg(target_os = "linux")]
 pub mod unshare;
 
+#[derive(Debug)]
+/// Errors related to image operations (downloading, caching, etc.)
+pub enum ImageError {
+    /// Cached image not found and downloading is not allowed
+    CachedImageNotFound {
+        /// Path where the image was expected to be cached
+        path: std::path::PathBuf,
+    },
+    /// Download is not available (missing feature or other reason)
+    DownloadNotAvailable {
+        /// Reason why download is not available
+        reason: String,
+    },
+    /// Architecture not supported for cloud images
+    UnsupportedArchitecture {
+        /// The unsupported architecture
+        arch: String,
+    },
+    /// Failed to download cloud image
+    DownloadFailed {
+        /// URL that failed to download
+        url: String,
+        /// Error message describing the failure
+        error: String,
+    },
+}
+
 #[derive(Debug)]
 /// Errors that can occur in a session.
 pub enum Error {
@@ -20,6 +47,8 @@ pub enum Error {
     IoError(std::io::Error),
     /// Error from setting up the session, with a message and detailed description.
     SetupFailure(String, String),
+    /// Error from image operations (download, cache, etc.)
+    ImageError(ImageError),
 }
 
 impl From<std::io::Error> for Error {
@@ -28,12 +57,36 @@ impl From<std::io::Error> for Error {
     }
 }
 
+impl std::fmt::Display for ImageError {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        match self {
+            ImageError::CachedImageNotFound { path } => {
+                write!(
+                    f,
+                    "Cached image not found at {} and downloading is not allowed",
+                    path.display()
+                )
+            }
+            ImageError::DownloadNotAvailable { reason } => {
+                write!(f, "Download not available: {}", reason)
+            }
+            ImageError::UnsupportedArchitecture { arch } => {
+                write!(f, "Architecture {} not supported for cloud images", arch)
+            }
+            ImageError::DownloadFailed { url, error } => {
+                write!(f, "Failed to download from {}: {}", url, error)
+            }
+        }
+    }
+}
+
 impl std::fmt::Display for Error {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         match self {
             Error::CalledProcessError(code) => write!(f, "CalledProcessError({})", code),
             Error::IoError(e) => write!(f, "IoError({})", e),
             Error::SetupFailure(msg, _long_description) => write!(f, "SetupFailure({})", msg),
+            Error::ImageError(e) => write!(f, "ImageError: {}", e),
         }
     }
 }
@@ -773,3 +826,39 @@ mod tests {
         assert_eq!(output, vec!["Hello, world!"]);
     }
 }
+
+/// Test utilities for sessions
+#[cfg(test)]
+pub mod test_utils {
+    /// Get a test session for use in tests.
+    ///
+    /// This returns an isolated session that's suitable for testing.
+    /// On Linux, it tries to create an UnshareSession for better isolation.
+    /// If that fails (e.g., no unshare permissions), it falls back to PlainSession.
+    /// The session is isolated from the host system when possible.
+    ///
+    /// Returns None only if no session can be created at all.
+    #[cfg(target_os = "linux")]
+    pub fn get_test_session() -> Option<Box<dyn super::Session>> {
+        // In CI environments like GitHub Actions, skip UnshareSession due to permission restrictions
+        if std::env::var("GITHUB_ACTIONS").is_ok() {
+            return Some(Box::new(super::plain::PlainSession::new()));
+        }
+
+        // Try to create an UnshareSession for isolation
+        if let Ok(session) = super::unshare::UnshareSession::bootstrap() {
+            return Some(Box::new(session));
+        }
+
+        // Fall back to PlainSession if unshare isn't available
+        Some(Box::new(super::plain::PlainSession::new()))
+    }
+
+    /// Get a test session for use in tests (non-Linux fallback).
+    ///
+    /// On non-Linux systems, returns a PlainSession for testing.
+    #[cfg(not(target_os = "linux"))]
+    pub fn get_test_session() -> Option<Box<dyn super::Session>> {
+        Some(Box::new(super::plain::PlainSession::new()))
+    }
+}
